package fr.nitorac.aurionweb.fragments;

import android.content.Context;
import android.graphics.Color;
import android.net.http.SslCertificate;
import android.net.http.SslError;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.webkit.CookieManager;
import android.webkit.SslErrorHandler;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.webkit.WebSettingsCompat;
import androidx.webkit.WebViewFeature;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.security.cert.CertificateException;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import fr.nitorac.aurionweb.R;
import fr.nitorac.aurionweb.Utils;
import fr.nitorac.aurionweb.aurionweb.AurionManager;
import fr.nitorac.aurionweb.aurionweb.AurionSession;
import fr.nitorac.aurionweb.aurionweb.CheckConnectionTask;
import fr.nitorac.aurionweb.security.SecurityManager;

public abstract class AurionViewFragment extends Fragment {

    protected AurionSession currentSession = new AurionSession();
    protected WebView webView;
    private View rootView;

    public abstract void preload();

    public void pageFinished(WebView view, String url){
        //Overridable empty func
    }

    public void networkInit(AurionSession session){
        //Overridable empty func
    }

    protected void initLoad(){
        CheckConnectionTask check = new CheckConnectionTask(getContext(), currentSession, res -> {
            if(res){
                setCookie();
                preload();
            }else{
                AurionManager.getInstance().spawnLoginDialog(getContext(), (ignored) ->{
                    currentSession.clear();
                    initLoad();
                });
            }
        }, this::networkInit);
        check.execute(SecurityManager.creds);
    }

    public View onCreateView(@NonNull LayoutInflater inflater,
                             ViewGroup container, Bundle savedInstanceState) {
        if(rootView == null){
            rootView = inflater.inflate(R.layout.fragment_aurionview, container, false);

            webView = rootView.findViewById(R.id.aurionView);

            webView.getSettings().setSupportZoom(true);
            webView.getSettings().setJavaScriptEnabled(true);

            webView.setBackgroundColor(Color.BLACK);

            if (WebViewFeature.isFeatureSupported(WebViewFeature.FORCE_DARK)) {
                WebSettingsCompat.setForceDark(webView.getSettings(), WebSettingsCompat.FORCE_DARK_ON);
            }

            webView.setWebViewClient(new WebViewClient() {
                @Override
                public void onPageFinished(WebView view, String url) {
                    if(url.contains("cas.ensiie.fr")){
                        initLoad();
                        return;
                    }
                    applyDarkTheme();
                    pageFinished(view, url);
                    super.onPageFinished(view, url);
                }

                @Override
                public void onReceivedError(WebView view, int errorCode, String description, String failingUrl) {
                    Log.e("WebClient", description);
                }

                @Override
                public void onReceivedSslError(WebView view, SslErrorHandler handler, SslError er) {
                    Context c = AurionViewFragment.this.getContext();
                    try{
                        SslCertificate cas = new SslCertificate(Utils.getCasCert(c));
                        SslCertificate aurion = new SslCertificate(Utils.getAurionCert(c));
                        SslCertificate erCert = er.getCertificate();

                        if(Build.VERSION.SDK_INT > Build.VERSION_CODES.Q){
                            if(Objects.equals(aurion.getX509Certificate(), erCert.getX509Certificate()) ||
                                    Objects.equals(cas.getX509Certificate(), erCert.getX509Certificate())){
                                handler.proceed();
                            }else{
                                handler.cancel();
                            }
                        }else{
                            // Minimal verification but Android won't let me convert SslCertificate to Certificate
                            if((Objects.equals(cas.getIssuedBy(), erCert.getIssuedBy()) && Objects.equals(cas.getIssuedTo(), erCert.getIssuedTo()) && Objects.equals(cas.getValidNotAfterDate(), erCert.getValidNotAfterDate()) && Objects.equals(cas.getValidNotBeforeDate(), erCert.getValidNotBeforeDate())) ||
                                    (Objects.equals(aurion.getIssuedBy(), erCert.getIssuedBy()) && Objects.equals(aurion.getIssuedTo(), erCert.getIssuedTo()) && Objects.equals(aurion.getValidNotAfterDate(), erCert.getValidNotAfterDate()) && Objects.equals(aurion.getValidNotBeforeDate(), erCert.getValidNotBeforeDate()))){
                                handler.proceed();
                            }else{
                                handler.cancel();
                            }
                        }
                    }catch (CertificateException e){
                        e.printStackTrace();
                        Toast.makeText(c, "Erreur de certificat SSL : " + e.getLocalizedMessage(), Toast.LENGTH_LONG).show();
                        handler.cancel();
                    }
                }

            });

            initLoad();

            if(Build.VERSION.SDK_INT > Build.VERSION_CODES.LOLLIPOP) {
                CookieManager.getInstance().setAcceptThirdPartyCookies(webView, true);
                CookieManager.getInstance().removeAllCookies(bool -> {});
            }
        }

        return rootView;
    }

    @Override
    public void onResume() {
        super.onResume();
        setCookie();
    }

    public void applyDarkTheme(){
        if(getContext() == null){
            return;
        }

        try {
            BufferedReader br = new BufferedReader(new InputStreamReader(getContext().getResources().openRawResource(R.raw.dark_theme)));
            String woaw = br.readLine();
            webView.evaluateJavascript("$('head').append('<style type=\"text/css\" rel=\"stylesheet\">" + woaw + "</style>');", paRes -> {});
        } catch (IOException e) {
            e.printStackTrace();
            Toast.makeText(getContext(), "Impossible de récupérer le thème sombre (" + e.getLocalizedMessage() + ")", Toast.LENGTH_LONG).show();
        }
    }

    public void setCookie(){
        CookieManager.getInstance().setCookie("https://aurionweb.ensiie.fr", "JSESSIONID=" + currentSession.getJSESSIONID());
    }

    public Map<String, String> getHeaders(){
        HashMap<String, String> res = new HashMap<>();
        res.put("Accept", "*/*");
        res.put("Accept-Encoding", "gzip, deflate");
        res.put("Cache-Control", "no-cache");

        return res;
    }
}
