package fr.nitorac.aurionweb.security;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.SharedPreferences;
import android.security.KeyPairGeneratorSpec;
import android.util.Base64;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.nio.charset.StandardCharsets;
import java.security.GeneralSecurityException;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.KeyPairGenerator;
import java.security.KeyStore;
import java.security.SecureRandom;
import java.util.ArrayList;
import java.util.Calendar;

import javax.crypto.Cipher;
import javax.crypto.CipherInputStream;
import javax.crypto.CipherOutputStream;
import javax.crypto.spec.SecretKeySpec;
import javax.security.auth.x500.X500Principal;

import fr.nitorac.aurionweb.StorageManager;

public class SecurityHelperPreM implements SecurityHelper {

    private static final String AndroidKeyStore = "AndroidKeyStore";
    private static final String RSA_MODE =  "RSA/ECB/PKCS1Padding";
    private static final String AES_MODE = "AES/ECB/PKCS7Padding";

    private Context context;
    private KeyStore keyStore;

    public SecurityHelperPreM(Context context) throws GeneralSecurityException, IOException {
        this.context = context;
        init();
    }

    public void init() throws GeneralSecurityException, IOException {
        keyStore = KeyStore.getInstance(AndroidKeyStore);
        keyStore.load(null);// Generate the RSA key pairs
        if (!keyStore.containsAlias(SecurityManager.KEYSTORE_NAME)) {
            // Generate a key pair for encryption
            Calendar start = Calendar.getInstance();
            Calendar end = Calendar.getInstance();
            end.add(Calendar.YEAR, 60);
            KeyPairGeneratorSpec spec = new KeyPairGeneratorSpec.Builder(context)
                    .setAlias(SecurityManager.KEYSTORE_NAME)
                    .setSubject(new X500Principal("CN=" + SecurityManager.KEYSTORE_NAME))
                    .setSerialNumber(BigInteger.TEN)
                    .setStartDate(start.getTime())
                    .setEndDate(end.getTime())
                    .build();
            KeyPairGenerator kpg = KeyPairGenerator.getInstance("RSA", AndroidKeyStore);
            kpg.initialize(spec);
            kpg.generateKeyPair();
        }

        SharedPreferences pref = context.getSharedPreferences(StorageManager.SEC_SHARED_PREF, Context.MODE_PRIVATE);
        String enryptedKeyB64 = pref.getString(SecurityManager.ENCRYPT_KEY, null);
        if (enryptedKeyB64 == null) {
            byte[] key = new byte[16];
            SecureRandom secureRandom = new SecureRandom();
            secureRandom.nextBytes(key);
            byte[] encryptedKey = rsaEncrypt(key);
            enryptedKeyB64 = Base64.encodeToString(encryptedKey, Base64.DEFAULT);
            SharedPreferences.Editor edit = pref.edit();
            edit.putString(SecurityManager.ENCRYPT_KEY, enryptedKeyB64);
            edit.apply();
        }
    }

    private byte[] rsaEncrypt(byte[] secret) throws GeneralSecurityException, IOException {
        KeyStore.PrivateKeyEntry privateKeyEntry = (KeyStore.PrivateKeyEntry) keyStore.getEntry(SecurityManager.KEYSTORE_NAME, null);
        // Encrypt the text
        Cipher inputCipher = Cipher.getInstance(RSA_MODE, "AndroidOpenSSL");
        inputCipher.init(Cipher.ENCRYPT_MODE, privateKeyEntry.getCertificate().getPublicKey());

        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        CipherOutputStream cipherOutputStream = new CipherOutputStream(outputStream, inputCipher);
        cipherOutputStream.write(secret);
        cipherOutputStream.close();

        return outputStream.toByteArray();
    }

    private  byte[]  rsaDecrypt(byte[] encrypted) throws GeneralSecurityException, IOException {
        KeyStore.PrivateKeyEntry privateKeyEntry = (KeyStore.PrivateKeyEntry)keyStore.getEntry(SecurityManager.KEYSTORE_NAME, null);
        Cipher output = Cipher.getInstance(RSA_MODE, "AndroidOpenSSL");
        output.init(Cipher.DECRYPT_MODE, privateKeyEntry.getPrivateKey());
        CipherInputStream cipherInputStream = new CipherInputStream(
                new ByteArrayInputStream(encrypted), output);
        ArrayList<Byte> values = new ArrayList<>();
        int nextByte;
        while ((nextByte = cipherInputStream.read()) != -1) {
            values.add((byte)nextByte);
        }

        byte[] bytes = new byte[values.size()];
        for(int i = 0; i < bytes.length; i++) {
            bytes[i] = values.get(i);
        }
        return bytes;
    }

    private Key getSecretKey(Context context) throws GeneralSecurityException, IOException {
        SharedPreferences pref = context.getSharedPreferences(StorageManager.SEC_SHARED_PREF, Context.MODE_PRIVATE);
        String enryptedKeyB64 = pref.getString(SecurityManager.ENCRYPT_KEY, null);
        if(enryptedKeyB64 == null){
            throw new InvalidKeyException("Impossible de trouver la clé AES sauvegardée !");
        }
        byte[] encryptedKey = Base64.decode(enryptedKeyB64, Base64.DEFAULT);
        byte[] key = rsaDecrypt(encryptedKey);
        return new SecretKeySpec(key, "AES");
    }

    @SuppressLint("GetInstance") // Vu qu'on fait du legacy ici, ça sert à rien d'avoir ces warnings
    public String encrypt(String input) throws GeneralSecurityException, IOException {
        Cipher c = Cipher.getInstance(AES_MODE, "BC");
        c.init(Cipher.ENCRYPT_MODE, getSecretKey(context));
        byte[] encodedBytes = c.doFinal(input.getBytes(StandardCharsets.UTF_8));
        return Base64.encodeToString(encodedBytes, Base64.DEFAULT);
    }

    @SuppressLint("GetInstance") // Vu qu'on fait du legacy ici, ça sert à rien d'avoir ces warnings
    public String decrypt(String encrypted) throws GeneralSecurityException, IOException {
        Cipher c = Cipher.getInstance(AES_MODE, "BC");
        c.init(Cipher.DECRYPT_MODE, getSecretKey(context));
        return new String(c.doFinal(Base64.decode(encrypted, Base64.DEFAULT)), StandardCharsets.UTF_8);
    }
}
